//----------------------------------------------------------------------------
//
//  File:         e2mysql.cpp
//  Author:       Michael Benzinger
//  Created:      06/26/2002
//  Description:  Helper functions for ESQL -> MySQL preprocessor for use only
//                by ATSE Hybrid projects.
//
//                Based on original work by Alan Walker, Sabre Labs.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
// USA.
//
//----------------------------------------------------------------------------



#define DEFINE_MYSQL_VARIABLES
#include "e2mysql.h"

#include <iostream>
#include <iomanip>
#include <pthread.h>


#ifdef __hpux
  extern "C"
  {
    void U_STACK_TRACE();
  }
#endif


//#################################################################
//#####        Connection and Error Handling Functions        #####
//#################################################################
const char *server_argc[] = {
  NULL
};

const char *server_groups[] = {
  NULL
};


static bool serverInitialized = false;


int32_t
SQLHelperConnect(void)
{
  if ( ! serverInitialized )
  {
    mysql_server_init(1, (char**) server_argc, (char**) server_groups);
    serverInitialized = true;
  }

  if ( ! mysql_init(&mysqlConnection) )
  {
    std::cerr << "MySQL ERROR: Unable to initialize a database object." << std::endl;
    exit(1);
  }

  std::cout << "Connecting to database" << std::endl;

  std::string mysqlHost     = "abc.xyz.com";
  std::string mysqlPort     = "3606";
  std::string mysqlDB       = "database";
  std::string mysqlUser     = "user";
  std::string mysqlPassword = "secret";


  if ( ! mysql_real_connect(&mysqlConnection, mysqlHost.c_str(), mysqlUser.c_str(), mysqlPassword.c_str(), mysqlDB.c_str(), atoi(mysqlPort.c_str()), NULL, 0) )
  {
     std::cerr << "Connection to multi daemon mode database failed: " << mysqlDB << std::endl;
     std::cerr << mysql_error(&mysqlConnection) << std::endl;

     exit(1);
  }

  std::cout << "Connected: " << mysqlDB << std::endl;

  connectionMade = true;

  return 0;
}


void
SQLHelperDisconnect(void)
{
  mysql_close(&mysqlConnection);
  connectionMade = false;
}


void
handle_error(const char* msg)
{
  std::cerr << "MySQL Error: " << msg << std::endl;
  std::cerr << "MySQL Error: " << mysql_error(&mysqlConnection) << std::endl;

  throw std::runtime_error("SQL Error");
}


