//----------------------------------------------------------------------------
//
//  File:         e2odbc.cpp
//  Author:       Michael Benzinger - Sabre Holdings
//  Created:      06/26/2002
//  Description:  Helper functions for ESQL -> ODBC preprocessor for use only
//                by ATSE Hybrid projects.
//
//                Based on original work by Alan Walker, Sabre Labs.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
// USA.
//
//----------------------------------------------------------------------------


#if ! defined(__TANDEM) && defined(TimesTen_Database)

#define DEFINE_ODBC_VARIABLES
#ifndef   E2ODBC_H
#include "e2odbc.h"
#endif
#ifndef   INIPARAMS_H
#include "IniParams.h"
#endif

#ifndef   IOSTREAM_H
#include <iostream>
#define   IOSTREAM_H
#endif

#ifndef   IOMANIP_H
#include <iomanip>
#define   IOMANIP_H
#endif



#ifdef __hpux
  extern "C"
  {
    void U_STACK_TRACE();
  }
#endif


//#################################################################
//#####        Connection and Error Handling Functions        #####
//#################################################################
int32
SQLHelperConnect(void)
{
  atse::IniParams& iniHandler = atse::IniParams::getGlobal();
  std::string      connectionString;

  if ( ! iniHandler.isInitialized() )
    iniHandler.init();

  if ( ! iniHandler.find("Hybrid", "ConnectionString", connectionString) )
  {
    std::cerr << "  ERROR: Unable to locate Hybrid\\ConnectionString in atse.ini!" << std::endl;

    exit(1);
  }

  char connStr[1024];

  snprintf(connStr, sizeof(connStr), "%s", connectionString.c_str());

  try
  {
    std::cout << "Connecting to database" << std::endl;

    SQLRETURN rc = SQLAllocEnv(&odbcEnvironment);
    if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
      handle_error(odbcEnvironment, odbcConnection, SQL_NULL_HSTMT, rc, "SQLAllocEnv");

    rc = SQLAllocConnect(odbcEnvironment, &odbcConnection);
    if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
      handle_error(odbcEnvironment, odbcConnection, SQL_NULL_HSTMT, rc, "SQLAllocEnv");

    SQLCHAR     szConnStrOut[1024];
    SQLSMALLINT cbConnStrOut;

    rc = SQLDriverConnect(odbcConnection, NULL, (UCHAR*) connStr, SQL_NTS, szConnStrOut, sizeof(szConnStrOut), &cbConnStrOut, SQL_DRIVER_NOPROMPT);
    if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
      handle_error(odbcEnvironment, odbcConnection, SQL_NULL_HSTMT, rc, "SQLAllocEnv");

    std::cout << "Connected: " << szConnStrOut << std::endl;
  }

  catch(...)
  {
    std::cerr << "Unable to make connection. Exiting applcation." << std::endl;

    exit(1);
  }

  return 0;
}


#ifdef __sparcv9
static void
stackTrace()
{
  char buffer[1024];
  char filename[1024];
  char line[1024];

  // Create command for traceback and copy to /var/tmp/traceback-(pid).txt.
  pid_t pid = getpid();

  sprintf(filename, "/var/tmp/traceback-%d.txt",      static_cast<int>(pid));
  sprintf(buffer,   "/usr/proc/bin/pstack %d > %s\n", static_cast<int>(pid), filename);

  // Undefine LD_PRELOAD to avoid 64-bit problems and issue the command.
  putenv(const_cast<char*>("LD_PRELOAD="));
  system(buffer);

  // Stream the contents of the stack file to standard out.
  FILE* file = fopen(filename, "r");

  if ( file )
  {
    fgets(line, sizeof(line) - 1, file);

    while ( ! feof(file) )
    {
      printf("%s", line);

      fgets(line, sizeof(line) - 1, file);
    }
  }

  // Close and clean up the temporary file.
  fclose(file);
  remove(filename);
}
#endif


void
handle_error(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, int32 returnCode, const char* msg)
{
  const int32 maxMsgLen = 2048;
  SQLCHAR     szSqlState[maxMsgLen];
  SQLCHAR     szErrorMsg[maxMsgLen];
  SQLINTEGER  pfNativeError;
  SQLSMALLINT pcbErrorMsg;

  std::cerr << "Return code " << returnCode << " from " << msg << std::endl;

  RETCODE rc = SQLError(henv, hdbc, hstmt, szSqlState, &pfNativeError, szErrorMsg, maxMsgLen, &pcbErrorMsg);

  while ( rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO )
  {
    std::cerr << "  Message(" << std::setw(4) << pfNativeError << "): " << szErrorMsg << std::endl;

    rc = SQLError(henv, hdbc, hstmt, szSqlState, &pfNativeError, szErrorMsg, maxMsgLen, &pcbErrorMsg);
  }

#if defined(__sparcv9)
  stackTrace();
#elif defined(__hpux)
  U_STACK_TRACE();
#endif

  throw std::runtime_error("SQL Error");
}

#endif  // __TANDEM && TimesTen_Database


