#ifndef   E2ODBC_H
#define   E2ODBC_H

//----------------------------------------------------------------------------
//
//  File:         e2odbc.h
//  Author:       Michael Benzinger - Sabre Holdings
//  Created:      06/26/2002
//  Description:  Helper functions for ESQL -> ODBC preprocessor for use only
//                by ATSE Hybrid projects.
//
//                Based on original work by Alan Walker, Sabre Labs.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
// USA.
//
//----------------------------------------------------------------------------

// This file is only used for ODBC-based compiles.
#if ! defined(__TANDEM) && defined(TimesTen_Database)

#ifndef   STD_PORTING_H
#include "std_porting.h"
#endif

#ifdef _WIN32
  #ifndef   WINDOWS_H
  #include <windows.h>
  #define   WINDOWS_H
  #endif
#endif

#ifdef __sparcv9
  #ifndef   UNISTD_H
  #include <unistd.h>
  #define   UNISTD_H
  #endif
#endif

#ifndef   SQL_H
#include <sql.h>
#define   SQL_H
#endif

#ifndef   SQLTYPES_H
#include <sqltypes.h>
#define   SQLTYPES_H
#endif

#ifndef   SQLEXT_H
#include <sqlext.h>
#define   SQLEXT_H
#endif

#ifndef   STDIO_H
#include <stdio.h>
#define   STDIO_H
#endif

#ifndef   STDEXCEPT_H
#include <stdexcept>
#define   STDEXCEPT_H
#endif

#ifndef   TIMESTEN_H
#include <timesten.h>
#define   TIMESTEN_H
#endif


#ifndef   SYS_TIME_H
#include <sys/time.h>
#define   SYS_TIME_H
#endif



// Define ODBC variables.
#ifdef DEFINE_ODBC_VARIABLES
SQLHENV  odbcEnvironment = 0;
SQLHDBC  odbcConnection  = 0;
#else
extern SQLHENV  odbcEnvironment;
extern SQLHDBC  odbcConnection;
#endif


static SQLLEN dummyNullInd;
static int16  sqlcode;


// Define structure to hold SQL statement information.
struct e2odbc
{
  const char* text;
  bool        bind;
  SQLHSTMT    stmt;
};


// SQLCA and SQLSA definitions for compatibility with Tandem Himalaya
static struct sqlca_struct
{
  int sqlcode;
  int fileCode;
  int misc;
} sqlca;

#define SQLCAFSCODE(_parm) (sqlca.fileCode)
define SQLCATOBUFFER(x,y,z) snprintf(y, z, "%hd", *x)
#define TESTSQLCODE(_parm)


//#################################################################
//#####        Connection and Error Handling Functions        #####
//#################################################################
int32
SQLHelperConnect(void);

void
handle_error(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt, int32 returnCode, const char *msg);


//#################################################################
//#####          Tandem Specific Function Emulations          #####
//#################################################################
inline int32
BEGINTRANSACTION(long* txnID)
{
  return 0;
}

inline int32
ENDTRANSACTION()
{
  return SQLTransact(odbcEnvironment, odbcConnection, SQL_COMMIT);
}

inline int32
RESUMETRANSACTION(long txnID)
{
  return 0;
}

inline int32
ABORTTRANSACTION()
{
  return SQLTransact(odbcEnvironment, odbcConnection, SQL_ROLLBACK);
}

inline int64
getJulianTimeStamp()
{
  static const int64 microsecondsToUNIXEpoch = 210866760000000000;

  struct timeval timeOfDay;
  gettimeofday(&timeOfDay, NULL);

  struct tm* pt = localtime(&timeOfDay.tv_sec);

  return (static_cast<int64>(mktime(pt)) * 1000000LL + timeOfDay.tv_usec) + microsecondsToUNIXEpoch;
}

//#################################################################
//#####          Bind Host Parameters for the Query           #####
//#################################################################
inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const char&   parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: char");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const char*   parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, bufLen, 0, (void*)  parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: char*");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const int16&  parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_SSHORT, SQL_INTEGER, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: int16");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const uint16& parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_USHORT, SQL_INTEGER, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: uint16");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const int32&  parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: int32");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const uint32& parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_ULONG, SQL_INTEGER, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: uint32");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const int64&  parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_SBIGINT, SQL_INTEGER, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: int64");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const float&  parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_FLOAT, SQL_FLOAT, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: float");
  return rc;
}

inline int32
SQLBindParmPoly(SQLHSTMT hstmt, SQLUSMALLINT col, const double& parmPtr, SQLINTEGER bufLen)
{
  int32 rc = SQLBindParameter(hstmt, col, SQL_PARAM_INPUT, SQL_C_DOUBLE, SQL_DOUBLE, bufLen, 0, (void*) &parmPtr, bufLen, 0);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindParameter: double");
  return rc;
}


//#################################################################
//#####          Bind Host Variables to Accept Data           #####
//#################################################################
inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, char& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_CHAR, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: char");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, char* tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_CHAR, tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: char*");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, int16& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_SSHORT, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: int16");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, uint16& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_USHORT, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: uint16");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, int32& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_SLONG, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: int32");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, uint32& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_ULONG, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: uint32");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, int64& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_SBIGINT, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: int64");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, float&  tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_FLOAT, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: float");
  return rc;
}

inline int32
SQLBindColPoly(SQLHSTMT hstmt, SQLUSMALLINT col, double& tgtPtr, SQLINTEGER bufLen, SQLLEN *ind)
{
  int32 rc = SQLBindCol(hstmt, col, SQL_C_DOUBLE, &tgtPtr, bufLen, ind);
  if ( rc == SQL_ERROR || rc == SQL_INVALID_HANDLE )
    handle_error(odbcEnvironment, odbcConnection, hstmt, rc, "SQLBindCol: double");
  return rc;
}


//#################################################################
//#####             Rollback and Commit Functions             #####
//#################################################################
inline void
esqlRollback()
{
  try
  {
    // EXEC SQL ROLLBACK
    sqlcode = SQLTransact(odbcEnvironment, odbcConnection, SQL_ROLLBACK);
    if ( sqlcode == SQL_ERROR || sqlcode == SQL_INVALID_HANDLE )
      handle_error(odbcEnvironment, odbcConnection, SQL_NULL_HSTMT, sqlcode, "SQLTransact");
  }
  catch(...)
  {
  }
}


inline void
esqlCommit()
{
  try
  {
    // EXEC SQL COMMIT
    sqlcode = SQLTransact(odbcEnvironment, odbcConnection, SQL_COMMIT);
    if ( sqlcode == SQL_ERROR || sqlcode == SQL_INVALID_HANDLE )
      handle_error(odbcEnvironment, odbcConnection, SQL_NULL_HSTMT, sqlcode, "SQLTransact");
  }
  catch(...)
  {
  }
}


#endif  // __TANDEM && TimesTen_Database



#endif                                 //  E2ODBC_H


